/*
 * nextion_uart.c
 *
 *  Created on: Jul 11, 2022
 *      Author: hussam
 */

#include "stm32f4xx.h"
#include "nextion_uart.h"
#include "stdio.h"
#include "string.h"
static void uart_set_baudrate(USART_TypeDef *USARTx, uint32_t PeriphClk,  uint32_t BaudRate);
#define AF07 0x07
uint8_t pwm_value;
#define NEXTION_BUFF_LENG 10
volatile char NEXTION_BUFF[NEXTION_BUFF_LENG];
volatile uint32_t itr;
volatile uint8_t end_of_data;
char data_srting[20];
uint8_t comend[3]={0xFF,0xFF,0xFF};

void nextion_uar_init(uint32_t baud,uint32_t freq)
{
	/*Enable clock access to GPIOA and USART1*/
	RCC->APB2ENR|=RCC_APB2ENR_USART1EN;
	RCC->AHB1ENR|=RCC_AHB1ENR_GPIOAEN;
	/*Configure the GPIO for UART Mode*/
	GPIOA->MODER|=GPIO_MODER_MODE9_1;
	GPIOA->MODER&=~GPIO_MODER_MODE9_0;
	GPIOA->MODER|=GPIO_MODER_MODE10_1;
	GPIOA->MODER&=~GPIO_MODER_MODE10_0;
	GPIOA->AFR[1]|=(AF07<<GPIO_AFRH_AFSEL9_Pos)|(AF07<<GPIO_AFRH_AFSEL10_Pos); //ALT7 for UART1 (PA9 and PA10)
	/*Configure UART*/
	uart_set_baudrate(USART1,freq,baud);
	USART1->CR1|=USART_CR1_TE|USART_CR1_RE;
	USART1->CR1|=USART_CR1_RXNEIE|USART_CR1_IDLEIE;
	NVIC_EnableIRQ(USART1_IRQn);
	USART1->CR1|=USART_CR1_UE;
}




static uint16_t compute_uart_bd(uint32_t PeriphClk, uint32_t BaudRate)
{
	return ((PeriphClk + (BaudRate/2U))/BaudRate);
}

static void uart_set_baudrate(USART_TypeDef *USARTx, uint32_t PeriphClk,  uint32_t BaudRate)
{
	USARTx->BRR =  compute_uart_bd(PeriphClk,BaudRate);
}


static void uart1_write(char ch)
{
  /*Make sure the transmit data register is empty*/
	while(!(USART1->SR & USART_SR_TXE)){}

  /*Write to transmit data register*/
	USART1->DR	=  (ch );
}





static void send_uart (char *str)
{
	while (*str) uart1_write (*str++);
}



void nextion_write_text(char *id,char * p)
{
	char buf[50];
	sprintf(buf,"%s.txt=\"%s\"",id,p);
	send_uart(buf);
	uart1_write(0xFF);
	uart1_write(0xFF);
	uart1_write(0xFF);

}

void nextion_plot_graph(uint16_t id ,uint16_t channel, uint8_t value)
	{
	char buff[50];
	sprintf(buff,"add %d,%d,%d",id,channel,value);
	send_uart(buff);
	uart1_write(0xFF);
	uart1_write(0xFF);
	uart1_write(0xFF);
	}


uint8_t received_data()
	{
	return end_of_data;
	}


void process_data()
{
	if(NEXTION_BUFF[0]==0x24) /*LED control*/
		{
			if(NEXTION_BUFF[2]==0x03) /*LED 1*/
			{
				//if(NEXTION_BUFF[3]==0x00){led1_off();nextion_write_text("t0","LED1 is off");}
				//if(NEXTION_BUFF[3]==0x01){led1_on();nextion_write_text("t0","LED1 is on");}
			}

			if(NEXTION_BUFF[2]==0x04) /*LED 2*/
			{
				//if(NEXTION_BUFF[3]==0x00){led2_off();nextion_write_text("t1","LED2 is off");}
				//if(NEXTION_BUFF[3]==0x01){led2_on();nextion_write_text("t1","LED2 is on");}
			}

		}

	if(NEXTION_BUFF[0]==0x40) /*Slider */
		{

			if(NEXTION_BUFF[2]==0x02) /*First slider*/
			{
				pwm_value=NEXTION_BUFF[3];
				Set_duty(LED0,pwm_value);
				sprintf(data_srting,"LED0 Duty %d",pwm_value);
				nextion_write_text("t0",data_srting);

			}
			if(NEXTION_BUFF[2]==0x04) /*First slider*/
			{
				pwm_value=NEXTION_BUFF[3];
				Set_duty(LED1,pwm_value);
				sprintf(data_srting,"LED1 Duty %d",pwm_value);
				nextion_write_text("t1",data_srting);
			}
		}

	memset(NEXTION_BUFF,0,NEXTION_BUFF_LENG);
	itr=0;
	end_of_data=0;

}



void USART1_IRQHandler(void)
{
    if (USART1->SR & USART_SR_RXNE) { /*If the interrupt source
    the RX-Line fill the buffer*/
    	NEXTION_BUFF[itr++]=USART1->DR;
    }
    /*If it is from IDLE line, set the end of data to 1 and read DR register*/
    if(USART1->SR & USART_SR_IDLE)
    	{
    		end_of_data=1;
    		(void)USART1->DR;
    	}
    if(itr==NEXTION_BUFF_LENG){

    	itr=0;
    }
}


